#!/bin/bash

# Author : epithetic

function MissingFiles {
	echo -e "\nPlease make sure the unip folders and this script are extracted to your CaveStory+ root directory (same folder as the game executable file)."
	exit
}


clear
echo "---------------------------------------"
echo "--- AGTP+ for CaveStory+ installer ----"
echo -e "---------------------------------------\n"

install=0
extract=0

# Check if installers and game files present

if [[ ! -d "agtp+4E-unip" ]] || [[ ! -d "agtp+-unip" ]] || [[ ! -d "classic+4E-unip" ]] || [[ ! -d "classic+-unip" ]] || [[ ! -d "sw_apo-unip" ]] || [[ ! -d "data" ]]; then MissingFiles; fi

if [[ ! -f "CaveStory+.exe" ]]; then
	if [[ ! -f "CS+.exe" ]]; then
		MissingFiles
	fi
fi

# Installation choices

./run-unip.sh s agtp+-unip
./run-unip.sh s agtp+4E-unip

echo -e "Please choose which version of the game you'd like to play:\n"

echo "1. Default CaveStory with AGTP+ translation"
echo "2. CaveStory including HaydenStudios' 4 Endings mod with AGTP+ translation"
while true; do
	read choice
	case "$choice" in
		[1]* ) version="default"; gamestring="AGTP+"; break;;
		[2]* ) version="4E"; gamestring="AGTP+4E"; break;;
		* ) echo "Please select 1 or 2.";;
	esac
done


clear
echo "---------------------------------------"
echo "--- AGTP+ for CaveStory+ installer ----"
echo -e "---------------------------------------\n"


./run-unip.sh s classic+-unip

echo -e "Please choose which graphics you'd like to play with:\n"

echo "1. CaveStory+ default graphics"
echo "2. Enhanced original CaveStory graphics"
while true; do
	read choice
	case "$choice" in
		[1]* ) graphics="default"; graphicsstring="default graphics"; break;;
		[2]* ) graphics="classic+"; graphicsstring="classic+ graphics"; break;;
		* ) echo "Please select 1 or 2.";;
	esac
done


clear
echo "---------------------------------------"
echo "--- AGTP+ for CaveStory+ installer ----"
echo -e "---------------------------------------\n"


./run-unip.sh s sw_apo-unip

echo -e "Please choose whether you prefer single width apostrophes:\n"

echo "1. Use default CaveStory+ font"
echo "2. Use single width apostrophes"
while true; do
	read choice
	case "$choice" in
		[1]* ) swapo="0"; swstring=""; break;;
		[2]* ) swapo="1"; swstring=" and single width apostrophes"; break;;
		* ) echo "Please select 1 or 2.";;
	esac
done


clear
echo "---------------------------------------"
echo "--- AGTP+ for CaveStory+ installer ----"
echo -e "---------------------------------------\n"

# Comment out extract only as some combinations would fail due to dependencies 

echo -e "Please choose whether to install, extract, or both:\n"

echo "1. Install these patches to my CaveStory+ installation" 
echo "2. Install these patches and extract them as plain files for local backup"
# echo "3. Only extract these patches as plain files for local backup"
while true; do
	read choice
	case "$choice" in
		[1]* ) install=1; iestring="install"; break;;
		[2]* ) extract=1; install=1; iestring="install and extract"; break;;
		#[3]* ) extract=1; iestring="extract"; break;;
		* ) echo "Please select 1 or 2.";;
	esac
done


clear
echo "---------------------------------------"
echo "--- AGTP+ for CaveStory+ installer ----"
echo -e "---------------------------------------\n"

echo -e "You have chosen to $iestring $gamestring with $graphicsstring$swstring.\n"


while true; do
		read -p "Is this correct? [Y/N]" yn
		case "$yn" in
		[Yy]* ) break;;
		[Nn]* ) exit;;
		* ) echo "Please answer yes or no.";;
	esac
done


# Chain all the installs now

# AGTP+ section

if [[ "$version" == 'default' ]]; then
	if [[ "$extract" -eq 1 ]]; then
		./run-unip.sh extract agtp+-unip
		read -p "Press any key to continue"
	fi
	if [[ "$install" -eq 1 ]]; then
		./run-unip.sh install agtp+-unip
		read -p "Press any key to continue"
	fi
fi

if [[ "$version" == '4E' ]]; then
	if [[ "$extract" -eq 1 ]]; then
		./run-unip.sh extract agtp+4E-unip
		read -p "Press any key to continue"
	fi
	if [[ "$install" -eq 1 ]]; then
		./run-unip.sh install agtp+4E-unip
		read -p "Press any key to continue"
	fi
fi


# Classic+ section

if [[ "$graphics" == 'classic+' ]]; then
	if [[ "$version" == 'default' ]]; then
		if [[ "$extract" -eq 1 ]]; then
			./run-unip.sh extract classic+-unip
			read -p "Press any key to continue"
		fi
		if [[ "$install" -eq 1 ]]; then
			./run-unip.sh install classic+-unip
			read -p "Press any key to continue"
		fi
	fi
	if [[ "$version" == '4E' ]]; then
		if [[ "$extract" -eq 1 ]]; then
			./run-unip.sh extract classic+4E-unip
			read -p "Press any key to continue"
		fi
		if [[ "$install" -eq 1 ]]; then
			./run-unip.sh install classic+4E-unip
			read -p "Press any key to continue"
		fi
	fi
fi


# Single-width apostrophe section

if [[ "$swapo" -eq 1 ]]; then
	if [[ "$extract" -eq 1 ]]; then
		./run-unip.sh extract sw_apo-unip
		read -p "Press any key to continue"
	fi
	if [[ "$install" -eq 1 ]]; then
		./run-unip.sh install sw_apo-unip
		read -p "Press any key to continue"
	fi
fi


echo -e "\nFinished installing selection."
exit
